@@ -46,6 +46,8 @@ static const NSString *kDisableSharingCalendar = @"Calendar";
 - (int) vmemLimit;
 - (BOOL) trustProxyAuthentication;
 - (NSString *) encryptionKey;
+- (BOOL) isSogoSecretSet;
+- (NSString *) sogoSecretValue;
 - (BOOL) useRelativeURLs;
 - (NSString *) sieveFolderEncoding;
 
@@ -78,6 +80,8 @@ static const NSString *kDisableSharingCalendar = @"Calendar";
 - (NSString *) pageTitle;
 - (NSString *) helpURL;
 
+NSComparisonResult languageSort(id el1, id el2, void *context);
+
 - (NSArray *) supportedLanguages;
 - (NSString *) loginSuffix;
 
@@ -128,6 +132,14 @@ static const NSString *kDisableSharingCalendar = @"Calendar";
 - (NSString *) JWTSecret;
 
 - (NSArray *) disableSharing;
+- (NSArray *) disableSharingAnyAuthUser;
+- (NSArray *) disableExport;
+
+- (BOOL) enableGlobalAddressBookFirstEntries;
+- (int) globalAddressBookFirstEntriesCount;
+
+- (BOOL)isURLEncryptionEnabled;
+- (NSString *)urlEncryptionPassphrase;
 
 @end
 
