@@ -41,6 +41,7 @@
 @class NSString;
 
 @class WOContext;
+@class WORequest;
 
 @class SOGoAppointmentFolder;
 @class SOGoAppointmentFolders;
@@ -135,6 +136,10 @@
 - (SOGoAppointmentFolder *) personalCalendarFolderInContext: (WOContext *) context;
 - (SOGoContactFolder *) personalContactsFolderInContext: (WOContext *) context;
 
+/* Encryption */
++ (NSString *)getEncryptedUsernameIfNeeded:(NSString *)username request:(WORequest *)request;
++ (NSString *)getDecryptedUsernameIfNeeded:(NSString *)username request:(WORequest *)request;
+
 @end
 
 #endif /* __SOGoUser_H__ */
