@@ -30,6 +30,7 @@
       this.$alwaysShowDetailedRecipients = (!stateMessage.to || stateMessage.to.length < 5) && (!stateMessage.cc || stateMessage.cc.length < 5);
       this.$showDetailedRecipients = this.$alwaysShowDetailedRecipients;
       this.showRawSource = false;
+      this.mailInDeletion = -1;
 
       _registerHotkeys(hotkeys);
 
@@ -98,6 +99,11 @@
         _.forEach(hotkeys, function(key) {
           sgHotkeys.deregisterHotkey(key);
         });
+        // Cancel automatic mark as read
+        if (vm.message.$markAsReadPromise)
+          vm.service.$timeout.cancel(vm.message.$markAsReadPromise);
+        // Remove controller from window
+        delete $window.$messageController;
       });
 
     }; // $onInit
@@ -148,7 +154,8 @@
         keys.push(sgHotkeys.createHotkey({
           key: hotkey,
           callback: _unlessInDialog(function($event) {
-            if (vm.mailbox.selectedCount() === 0)
+            if (vm.mailbox.selectedCount() === 0 
+                  && vm.message.uid !== vm.mailInDeletion) //prevent calling function if it was alredy made for this mail
               vm.deleteMessage();
             $event.preventDefault();
           }),
@@ -286,7 +293,7 @@
               $mdToast.show(
                 $mdToast.simple()
                   .textContent(l('Successfully created card'))
-                  .position('top right')
+                  .position(sgConstant.toastPosition)
                   .hideDelay(2000));
             });
           });
@@ -330,6 +337,7 @@
       if (Mailbox.$virtualMode) {
         mailbox = Mailbox.selectedFolder; // the VirtualMailbox instance
       }
+      vm.mailInDeletion = message.uid;
 
       function _success(index) {
         var nextIndex = index;
@@ -476,6 +484,12 @@
       }
     };
 
+    this.compose = function($event) {
+      if (!this._showMailEditorInPopup('compose')) {
+        _showMailEditor($event, this.message.$compose());
+      }
+    };
+
     this.openInPopup = function(action) {
       var url = [sgSettings.baseURL(),
                  'UIxMailPopupView#!/Mail',
@@ -485,7 +499,10 @@
                  this.message.uid]
           .join('/'),
           wId = this.message.$absolutePath();
-      if (action) url += '/' + action;
+      if (action) {
+        wId += '/' + action;
+        url += '/' + action;
+      }
       popupWindow = $window.open(url, wId,
                                  ["width=680",
                                   "height=520",
