@@ -24,6 +24,7 @@
         '    <div class="sg-tile-date"><!-- date --></div>',
         '  </div>',
         '  <div class="sg-md-body">',
+        '    <div class="sg-category-dot-container"><!-- categories --></div>',
         '    <div class="sg-tile-subject"><!-- subject --></div>',
         '    <div class="sg-tile-size"><!-- size --></div>',
         '    <md-button class="sg-tile-btn md-secondary ng-hide" md-colors="::{ color: \'accent-600\'}" ng-click="$ctrl.toggleThread()">',
@@ -57,8 +58,8 @@
   /**
    * @ngInject
    */
-  sgMessageListItemMainController.$inject = ['$scope', '$element', '$parse', '$state', '$mdUtil', '$mdToast', 'Mailbox', 'Message', 'encodeUriFilter'];
-  function sgMessageListItemMainController($scope, $element, $parse, $state, $mdUtil, $mdToast, Mailbox, Message, encodeUriFilter) {
+  sgMessageListItemMainController.$inject = ['$scope', '$element', '$parse', '$state', '$mdUtil', '$mdToast', 'Mailbox', 'Message', 'encodeUriFilter', 'Preferences'];
+  function sgMessageListItemMainController($scope, $element, $parse, $state, $mdUtil, $mdToast, Mailbox, Message, encodeUriFilter, Preferences) {
     var $ctrl = this;
 
     this.$postLink = function () {
@@ -82,7 +83,6 @@
       threadButton = angular.element(threadButton);
       this.threadIconElement = threadButton.find('md-icon')[0];
       this.threadCountElement = threadButton.find('span')[0];
-
       this.priorityIconElement = contentDivElement.find('md-icon')[0];
 
       if (Mailbox.$virtualMode) {
@@ -120,70 +120,73 @@
         var i;
         $ctrl.message = $ctrl.parentController.message;
 
-        // Flags
-        var flagElements = $mdUtil.nodesToArray($element[0].querySelectorAll('.sg-category'));
-        _.forEach(flagElements, function(flagElement) {
-          $element[0].removeChild(flagElement);
-        });
-        for (i = 0; i < $ctrl.message.flags.length && i < 5; i++) {
-          var tag = $ctrl.message.flags[i];
-          if ($ctrl.service.$tags[tag]) {
-            var flagElement = angular.element('<div class="sg-category"></div>');
-            flagElement.css('left', (i*3) + 'px');
-            flagElement.css('background-color', $ctrl.service.$tags[tag][1]);
-            $element.prepend(flagElement);
+        if (!$ctrl.message.loading) {
+          // Flags
+          var flagList = $element[0].querySelector('.sg-category-dot-container'),
+              $flagList = angular.element(flagList),
+              flagElements = $mdUtil.nodesToArray(flagList.querySelectorAll('.sg-category-dot'));
+          _.forEach(flagElements, function(flagElement) {
+            flagList.removeChild(flagElement);
+          });
+          for (i = 0; i < $ctrl.message.flags.length && i < 5; i++) {
+            var tag = $ctrl.message.flags[i];
+            if ($ctrl.service.$tags[tag]) {
+              var flagElement = angular.element('<div class="sg-category-dot"></div>');
+              flagElement.css('background-color', $ctrl.service.$tags[tag][1]);
+              $flagList.append(flagElement);
+            }
           }
-        }
 
-        // Mailbox name when in virtual mode
-        if ($ctrl.mailboxNameElement)
-          $ctrl.mailboxNameElement.innerHTML = $ctrl.message.$mailbox.$displayName;
+          // Mailbox name when in virtual mode
+          if ($ctrl.mailboxNameElement)
+            $ctrl.mailboxNameElement.innerHTML = $ctrl.message.$mailbox.$displayName;
 
-        // Sender or recipient when in
-        if ($ctrl.MailboxService.selectedFolder.type == 'sent')
-          $ctrl.senderElement.innerHTML = $ctrl.message.$shortAddress('to').encodeEntities();
-        else
-          $ctrl.senderElement.innerHTML = $ctrl.message.$shortAddress('from').encodeEntities();
-
-        // Priority icon
-        if ($ctrl.message.priority && $ctrl.message.priority.level < 3) {
-          $ctrl.priorityIconElement.classList.remove('ng-hide');
-          if ($ctrl.message.priority.level < 2)
-            $ctrl.priorityIconElement.classList.add('md-warn');
+          // Sender or recipient when in Sent or Draft mailbox
+          if ($ctrl.MailboxService.selectedFolder.isSentFolder || $ctrl.MailboxService.selectedFolder.isDraftsFolder)
+            $ctrl.senderElement.innerHTML = $ctrl.message.$shortAddress('to', Preferences.defaults.SOGoMailDisplayFullEmail).encodeEntities();
           else
-            $ctrl.priorityIconElement.classList.remove('md-warn');
+            $ctrl.senderElement.innerHTML = $ctrl.message.$shortAddress('from', Preferences.defaults.SOGoMailDisplayFullEmail).encodeEntities();
+
+          // Priority icon
+          if ($ctrl.message.priority && $ctrl.message.priority.level < 3) {
+            $ctrl.priorityIconElement.classList.remove('ng-hide');
+            if ($ctrl.message.priority.level < 2)
+              $ctrl.priorityIconElement.classList.add('md-warn');
+            else
+              $ctrl.priorityIconElement.classList.remove('md-warn');
+          }
+          else
+            $ctrl.priorityIconElement.classList.add('ng-hide');
+
+          // Mail thread
+          if ($ctrl.message.first) {
+            $ctrl.threadButton.classList.remove('ng-hide');
+            $ctrl.threadCountElement.innerHTML = $ctrl.message.threadCount;
+            if ($ctrl.message.collapsed)
+              $ctrl.threadIconElement.classList.remove('md-rotate-180-ccw');
+          }
+          else {
+            $ctrl.threadButton.classList.add('ng-hide');
+          }
+
+          // Subject
+          $ctrl.subjectElement.innerHTML = $ctrl.message.subject.encodeEntities();
+
+          // Message size
+          $ctrl.sizeElement.innerHTML = $ctrl.message.size;
+
+          // Received Date
+          $ctrl.dateElement.innerHTML = $ctrl.message.relativedate;
+
+          setVisibility($ctrl.flagIconElement,
+                        $ctrl.message.isflagged);
+          setVisibility($ctrl.answerIconElement,
+                        $ctrl.message.isanswered);
+          setVisibility($ctrl.forwardIconElement,
+                        $ctrl.message.isforwarded);
+          setVisibility($ctrl.attachmentIconElement,
+                        $ctrl.message.hasattachment);
         }
-        else
-          $ctrl.priorityIconElement.classList.add('ng-hide');
-
-        // Mail thread
-        if ($ctrl.message.first) {
-          $ctrl.threadButton.classList.remove('ng-hide');
-          $ctrl.threadCountElement.innerHTML = $ctrl.message.threadCount;
-          if ($ctrl.message.collapsed)
-            $ctrl.threadIconElement.classList.remove('md-rotate-180-ccw');
-        }
-        else {
-          $ctrl.threadButton.classList.add('ng-hide');
-        }
-
-        // Subject
-        $ctrl.subjectElement.innerHTML = $ctrl.message.subject.encodeEntities();
-
-        // Message size
-        $ctrl.sizeElement.innerHTML = $ctrl.message.size;
-
-        // Received Date
-        $ctrl.dateElement.innerHTML = $ctrl.message.relativedate;
-
-        setVisibility($ctrl.flagIconElement,
-                       $ctrl.message.isflagged);
-        setVisibility($ctrl.answerIconElement,
-                       $ctrl.message.isanswered);
-        setVisibility($ctrl.forwardIconElement,
-                       $ctrl.message.isforwarded);
-        setVisibility($ctrl.attachmentIconElement,
-                       $ctrl.message.hasattachment);
 
         // Call original method on parent controller
         angular.bind($ctrl.parentController, parentControllerOnUpdate)();
